# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

"""Tests for OWSProxyGETTranscoder"""

import urlparse
import unittest

from inteproxy.transcoder import OWSProxyGETTranscoder


class TestOWSProxyGETTranscoder(unittest.TestCase):

    def test_username_password_substitution(self):

        class Transcoder(OWSProxyGETTranscoder):
            """
            An OWSProxyGETTranscoder that uses usernames/passwords
            defined in this test case
            """
            def get_password(self):
                return user, password


        test_users = [("user", "password",
                       "https://example.com/wfs",
                       "https://example.com/wfs?user=user&password=password"),
                      ("foo", "bar",
                       # base url with trailing ?
                       "https://example.com/wfs?",
                       "https://example.com/wfs?user=foo&password=bar"),
                      ("foo", "bar",
                       # base url with existing query
                       "https://example.com/wfs?REQUEST=GetCapabilities"
                       "&VERSION=1.1.0&SERVICE=WFS",
                       "https://example.com/wfs?REQUEST=GetCapabilities"
                       "&VERSION=1.1.0&SERVICE=WFS&user=foo&password=bar"),
                      ("some \xc3\xbcser&", "<pass&>word?",
                       "https://example.com/wfs",
                       "https://example.com/wfs?"
                       "user=some+%C3%BCser%26&password=%3Cpass%26%3Eword%3F"),
                      ]
        for user, password, baseurl, encodedurl in test_users:
            transcoder = Transcoder("GET", urlparse.urlsplit(baseurl))
            self.assertEquals(transcoder.get_url(), encodedurl)
